unit Extractor;

interface

uses
  Windows, PngImageLite;

function ExtractThumb(FileName: string; SizeX, SizeY: Integer): hBitmap;
function LoadPNGImage(F: TFileStream; MapLength: Integer; out ImageX, ImageY: Integer): hBitmap;
function LoadBITImage(F: TFileStream; MapLength: Integer; out ImageX, ImageY: Integer): hBitmap;

implementation

uses
  Types, SysUtils, Resizer, LeadTools;

{procedure SaveDebug(B: hBitmap; FN: string);
var TB: Graphics.TBitmap;
begin
  TB:=TBitmap.Create;
  TB.Handle:=B;
  TB.SaveToFile(FN);
  //TB.Free;
end;}

function ExtractThumb(FileName: string; SizeX, SizeY: Integer): hBitmap;
var
  F: TFileStream;
  Sig: array[0..3] of Char;
  MapLength, MapType: Cardinal;
  hdcScreen, hdcB, hdcBR: hDC;
  Brush: hBrush;
  B, BR: hBitmap;
  ScaleX, ScaleY, ImageX, ImageY: Integer;
begin
  Result:=0;

  try
    F:=TFileStream.Create(FileName, [fsmRead]);
    FillChar(Sig, SizeOf(Sig), 0);
    F.Read(Sig, 4);
    if(Sig[0]='W') and
      (Sig[1]='A') then
      begin
      F.Read(MapLength, 4);
      F.Read(MapType, 4);
      {
      MS:=TMemoryStream.Create;
      MS.CopyFrom(F, MapLength-4);
      MS.Position:=0;
      F.Free;

      P:=TPNGObject.Create;
      P.LoadFromStream(MS);
      MS.Free;
      }
      case MapType of
        1: B:=LoadBITImage(F, MapLength, ImageX, ImageY);
        2: Exit; //randomly-generated map;
        3: B:=LoadPNGImage(F, MapLength, ImageX, ImageY);
        else
          raise Exception.Create('Unknown map type ('+IntToStr(MapType)+')');
        end;
      end
    else
      begin
      F.Position:=40;
      F.Read(Sig, 4);
      if(Sig[0]='L') and
        (Sig[1]='T') and
        (Sig[2]='R') and
        (Sig[3]='I') then   // BIT file
        begin
        F.Position:=0; 
        B:=LoadBITImage(F, F.Size, ImageX, ImageY);
        end
      else
          raise Exception.Create('Unknown file type');
      end;
    F.Free;

    if B=0 then
      raise Exception.Create('Decoder returned null bitmap');
    //SaveDebug(B, 'D1.bmp');

    ScaleX:=SizeX;
    ScaleY:=SizeY;
    if ImageX>ImageY then
      ScaleY:=Round(ScaleX/ImageX * ImageY)
    else
      ScaleX:=Round(ScaleY/ImageY * ImageX);

    BR:=_ScaleBitmapInt(B, ScaleX, ScaleY);
    DeleteObject(B);

    hdcScreen := GetDC(0);
    hdcBR := CreateCompatibleDC(hdcScreen); 
    
    SelectObject(hdcBR, BR);

    B:=CreateBitmap(SizeX, SizeY, 4, 8, nil);
    hdcB := CreateCompatibleDC(hdcScreen); 
    SelectObject(hdcB, B);
    
    Brush:=CreateSolidBrush(0);
    FillRect(hdcB, Rect(0, 0, SizeX, SizeY), Brush);
    DeleteObject(Brush);

    Brush:=CreateSolidBrush($010101);
    FillRect(hdcB, Bounds((SizeX-ScaleX)div 2, (SizeY-ScaleY)div 2, ScaleX, ScaleY), Brush);
    DeleteObject(Brush);

    //BitBlt(hdcB, (SizeX-ScaleX)div 2, (SizeY-ScaleY)div 2, ScaleX, ScaleY, hdcBR, 0, 0, SRCCOPY);
    TransparentBlt(hdcB, (SizeX-ScaleX)div 2, (SizeY-ScaleY)div 2, ScaleX, ScaleY, hdcBR, 0, 0, ScaleX, ScaleY, 0);
    //SaveDebug(B, 'D3.bmp');

    ReleaseDC(0, hdcScreen);
    DeleteDC(hdcB);
    DeleteDC(hdcBR);
    DeleteObject(BR);

    Result:=B;
  except
    //on E: Exception do
    //  MessageBox(0, PChar(E.Message), nil, 0);
    end;
end;

function LoadPNGImage(F: TFileStream; MapLength: Integer; out ImageX, ImageY: Integer): hBitmap;
var
  P: TPNGObject;
  hdcScreen, hdcB: hDC;
  B: hBitmap;
begin
  //Result:=0;

  P:=TPNGObject.Create;
  P.LoadFromStream(F);
  
  hdcScreen := GetDC(0);
  hdcB := CreateCompatibleDC(hdcScreen); 
  ReleaseDC(0, hdcScreen);

  if hdcB=0 then 
    raise Exception.Create('CreateCompatibleDC fail');

  B:=CreateBitmap(P.Width, P.Height, 4, 8, nil);
  if B=0 then 
    raise Exception.Create('CreateBitmap fail');
  if SelectObject(hdcB, B)=0 then
    raise Exception.Create('SelectObject fail');
  
  ImageX:=P.Width;
  ImageY:=P.Height;

  P.Draw(hdcB, Rect(0, 0, P.Width, P.Height));
  P.Free;

  DeleteDC(hdcB);
  Result:=B;
end;

function LoadBITImage(F: TFileStream; MapLength: Integer; out ImageX, ImageY: Integer): hBitmap;
var
  DIB: THandle;
  B: hBitmap;
  L: L_Image;
  Buffer: array[0..696-1, 0..(1920 div 8)-1] of Byte;
  DIBinMemory, P: Pointer;
  I: Integer;
begin
  if not LeadToolsLoaded then
    raise Exception.Create('Load failure');

  //MessageBox(0, PChar(IntToStr(F.Position+40)+' '+IntToStr(MapLength-40)), nil, 0);
  FillChar(L, SizeOf(L), 0);
  I:=L_LoadFileOffset(F.Handle, F.Position+40, MapLength-40, L, 1, 0, 3, 0, 0);
  if I<=0 then
    raise Exception.Create('L_LoadFileOffset failed ('+IntToStr(I)+')');
  
  ImageX:=L.Width;
  ImageY:=L.Height;

  if(L.Width<>1920)or(L.Height<>696) then
    raise Exception.Create('Bad size');       // well Worms NEVER makes BITs of other sizes

  DIB := L_ConvertToDIB(L);
  if DIB=0 then
    raise Exception.Create('L_ConvertToDIB failed');
  L_FreeBitmap(L);

  DIBinMemory := GlobalLock(DIB);

  B := CreateBitmap(L.Width, L.Height, 1, 1, nil);     // Pointer(Integer(DIBinMemory)+40 + 2*SizeOf(TRGBQuad))
  if B=0 then
    raise Exception.Create('CreateBitmap failure');

  for I:=0 to 696-1 do
    begin
    P := Pointer(Integer(DIBinMemory)+40 + 2*SizeOf(TRGBQuad) + I*(1920 div 8));
    Move(P^, Buffer[695-I], 1920 div 8);
    end;

  GlobalUnlock(DIB);
  GlobalFree(DIB);
  DeleteObject(DIB);

  SetBitmapBits(B, SizeOf(Buffer), @Buffer);
  
  L_FreeBitmap(L);
  Result:=B;
end;

end.
