unit WAgameThumbnail;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  Windows, ActiveX, ComObj, ShWAgame_TLB, ShlObj2;

type
  TWAgameThumbnail = class(TTypedComObject, IWAgameThumbnail, IExtractImage, IExtractImage2, IPersistFile)
  private
    FFileName: string;
    FFileError: boolean;
    hThumb: hBitmap;
    function GetIsFileLocked: Boolean;
  protected
    {Declare IWAgameThumbnail methods here}
    // IExtractIcon
    function GetLocation(Buffer : PWideChar;
                         BufferSize : cardinal;
                         var Priority : cardinal;
                         const Size : TSize;
                         ColorDepth : cardinal;
                         var Flags : cardinal) : hresult; stdcall;
    function Extract(var BmpThumbnail : HBITMAP) : hresult; stdcall;
    function GetDateStamp(var DateStamp : TFILETIME) : hresult; stdcall;
    // IPersist
    function GetClassID(out classID: TCLSID): HResult; stdcall;
    // IPersistFile
    function IsDirty: HResult; stdcall;
    function Load(pszFileName: POleStr; dwMode: Longint): HResult; stdcall;
    function Save(pszFileName: POleStr; fRemember: BOOL): HResult; stdcall;
    function SaveCompleted(pszFileName: POleStr): HResult; stdcall;
    function GetCurFile(out pszFileName: POleStr): HResult; stdcall;
    public
      property FileName: String  read   FFileName;
      property FileLocked: Boolean read GetIsFileLocked;
    end;

  TWAgameThumbnailFactory = class(TTypedComObjectFactory)
  protected
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

implementation

uses
  SysUtils, ComServ, Registry, Extractor;

function IsInCD(Const FileName: String; Flags: Cardinal): Boolean;
begin
  Result := GetDriveType( PChar( ExtractFileDrive( FileName ) ) ) = Flags;
end;

{ TWAgameThumbnail }

function TWAgameThumbnail.GetIsFileLocked: Boolean; 
var 
  hFile: THandle; 
begin 
  hFile := CreateFile ( PChar(FFileName), GENERIC_READ, FILE_SHARE_READ, nil, 
             OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0 ); 
  Result := hFile = INVALID_HANDLE_VALUE; 
  CloseHandle( hFile ); 
end;

function TWAgameThumbnail.GetLocation(Buffer : PWideChar;
                         BufferSize : cardinal;
                         var Priority : cardinal;
                         const Size : TSize;
                         ColorDepth : cardinal;
                         var Flags : cardinal) : hresult; stdcall; 
begin
  //MessageBoxW(0, Buffer, 'GetLocation', 0);
  //MessageBox(0, PChar('ColorDepth='+IntToStr(ColorDepth)), 'GetLocation', 0); 
  if (@Priority<>nil) then 
    Priority := IEI_PRIORITY_NORMAL;

  Result := NOERROR; 
  if (@Flags<>nil) then
    begin 
    //if (Flags and IEIFLAG_ASYNC) <> 0 then
    //  Result := E_PENDING;
    Flags := Flags + IEIFLAG_CACHE + IEIFLAG_REFRESH; 
    end; 

  hThumb:=ExtractThumb(FileName, Size.cX, Size.cY);
end;

function TWAgameThumbnail.Extract(var BmpThumbnail : HBITMAP) : hresult; stdcall;
begin
  //MessageBox(0, 'Extract...', '...', 0);
  BmpThumbnail := hThumb;
  Result := S_OK;
end;

function TWAgameThumbnail.GetDateStamp(var DateStamp : TFILETIME) : hresult; stdcall;
var
  hFile: THandle;
begin
  //MessageBox(0, 'gds', nil, 0);
  hFile := CreateFile(PChar(FileName), STANDARD_RIGHTS_READ, FILE_SHARE_READ,
      nil, OPEN_EXISTING, 0, 0);
  if hFile=0 then
    begin
    //MessageBox(0, 'Error in fopen', nil, 0);
    Result:=E_FAIL;
    Exit;
    end;
  GetFileTime(hFile, nil, nil, @DateStamp);
  CloseHandle(hFile);
  Result:=NOERROR;
end;

function TWAgameThumbnail.GetClassID(out classID: TGUID): HRESULT;
begin
  Result := E_NOTIMPL;
end;

function TWAgameThumbnail.GetCurFile(out pszFileName: PWideChar): HRESULT;
begin
  Result := E_NOTIMPL;
end;

function TWAgameThumbnail.IsDirty: HRESULT;
begin
  Result := E_NOTIMPL;
end;

function TWAgameThumbnail.Load(pszFileName: PWideChar; dwMode: Integer): HRESULT;
begin
  FFileName := pszFileName;
  Result := S_OK;

  FFileError := GetIsFileLocked;

  {
  if not Enabled then
    begin Result := E_FAIL; Exit; end; 

  if IsInCD( FFileName, DRIVE_CDROM ) then 
    if GetWorkOnDisc then 
      begin Result := E_FAIL; Exit; end; 

  if IsInCD( FFileName, DRIVE_REMOTE ) then 
    if GetWorkOnRemote then 
      begin Result := E_FAIL; Exit; end; 
  }
end; 

function TWAgameThumbnail.Save(pszFileName: PWideChar; fRemember: LongBool): HRESULT; 
begin 
  Result := E_NOTIMPL;
end;

function TWAgameThumbnail.SaveCompleted(pszFileName: PWideChar): HRESULT;
begin
  Result := E_NOTIMPL;
end; 

{ TWAgameThumbnailFactory } 

procedure RegisterWin32NT(Const ClassID, Description: String; Register: Boolean);
begin
  if (Win32Platform = VER_PLATFORM_WIN32_NT) then
    with TRegistry.Create do
      try
        RootKey := HKEY_LOCAL_MACHINE;
        OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Shell Extensions', True);
        OpenKey('Approved', True);
        if Register then
          WriteString(ClassID, Description)
        else
          DeleteValue(ClassID)
      finally
         Free;
      end;
end;

procedure RegisterBIT;
begin
  with TRegistry.Create do
    try
      RootKey := HKEY_CLASSES_ROOT;
      OpenKey('.bit', True);
      WriteString('', 'Worms.BIT');
      CloseKey;
      OpenKey('Worms.BIT', True);
      WriteString('', 'Worms Map File');
      CloseKey;
    finally
       Free;
    end;
end;

procedure TWAgameThumbnailFactory.UpdateRegistry(Register: Boolean);
var
  ClassID: string;
begin
  ClassID := GUIDToString( CLASS_WAgameThumbnail );
  if Register then
  begin
    inherited UpdateRegistry( Register );
    CreateRegKey('Team17.WAgame\shellex\{BB2E617C-0920-11d1-9A0B-00C04FC2D6C1}','',ClassID);
    RegisterBIT;
    CreateRegKey('Worms.BIT\shellex\{BB2E617C-0920-11d1-9A0B-00C04FC2D6C1}','',ClassID);
    RegisterWin32NT(ClassID, Description, Register);
  end
  else
  begin
    DeleteRegKey('Team17.WAgame\shellex\{BB2E617C-0920-11d1-9A0B-00C04FC2D6C1}');
    DeleteRegKey('Worms.BIT\shellex\{BB2E617C-0920-11d1-9A0B-00C04FC2D6C1}');
    RegisterWin32NT( ClassID, Description, Register);
    inherited UpdateRegistry( Register )
  end;
end;

initialization
  {TTypedComObjectFactory.Create(ComServer, TWAgameThumbnail, Class_WAgameThumbnail,
    ciMultiInstance, tmApartment);
  }
  TWAgameThumbnailFactory.Create(
    ComServer, TWAgameThumbnail, CLASS_WAgameThumbnail, ciMultiInstance, tmApartment
  );

  //MessageBox(0, 'Loaded...', nil, 0);
end.
